@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2005-2007 Imendio AB
- * Copyright (C) 2007 Collabora Ltd.
+ * Copyright (C) 2007-2008 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -45,10 +45,8 @@
 
 #include "empathy-accounts-dialog.h"
 #include "empathy-profile-chooser.h"
-#include "empathy-account-widget-generic.h"
-#include "empathy-account-widget-jabber.h"
-#include "empathy-account-widget-msn.h"
-#include "empathy-account-widget-salut.h"
+#include "empathy-account-widget.h"
+#include "empathy-account-widget-irc.h"
 
 #define DEBUG_DOMAIN "AccountDialog"
 
@@ -272,6 +270,18 @@ accounts_dialog_update_account (EmpathyAccountsDialog *dialog,
 			dialog->settings_widget =
 				empathy_account_widget_salut_new (account);
 		}
+		else if (!tp_strdiff (config_ui, "irc")) {
+			dialog->settings_widget =
+				empathy_account_widget_irc_new (account);
+		}
+		else if (!tp_strdiff(config_ui, "icq")) {
+			dialog->settings_widget =
+				empathy_account_widget_icq_new (account);
+		}
+		else if (!tp_strdiff (config_ui, "yahoo")) {
+			dialog->settings_widget =
+				empathy_account_widget_yahoo_new (account);
+		}
 		else {
 			dialog->settings_widget = 
 				empathy_account_widget_generic_new (account);
