@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2005-2007 Imendio AB
- * Copyright (C) 2007 Collabora Ltd.
+ * Copyright (C) 2007-2008 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -31,20 +31,22 @@
 #include <gtk/gtk.h>
 #include <glade/glade.h>
 
+#include <libtelepathy/tp-helpers.h>
+
 #include <libmissioncontrol/mc-account.h>
 #include <libmissioncontrol/mission-control.h>
 
 #include <libempathy/empathy-contact-factory.h>
 #include <libempathy/empathy-contact-list.h>
 #include <libempathy/empathy-log-manager.h>
+#include <libempathy/empathy-tp-group.h>
+#include <libempathy/empathy-contact-groups.h>
 #include <libempathy/empathy-debug.h>
 #include <libempathy/empathy-utils.h>
-#include <libempathy/empathy-marshal.h>
 
 #include "empathy-contact-list-view.h"
 #include "empathy-contact-list-store.h"
 #include "empathy-images.h"
-#include "empathy-contact-groups.h"
 #include "empathy-cell-renderer-expander.h"
 #include "empathy-cell-renderer-text.h"
 #include "empathy-cell-renderer-activatable.h"
@@ -53,6 +55,8 @@
 //#include "empathy-chat-invite.h"
 //#include "empathy-ft-window.h"
 #include "empathy-log-window.h"
+#include "empathy-gtk-enum-types.h"
+#include "empathy-gtk-marshal.h"
 
 #define DEBUG_DOMAIN "ContactListView"
 
@@ -65,12 +69,12 @@
 
 #define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), EMPATHY_TYPE_CONTACT_LIST_VIEW, EmpathyContactListViewPriv))
 
-struct _EmpathyContactListViewPriv {
-	EmpathyContactListStore *store;
-	GtkUIManager            *ui;
-	GtkTreeRowReference     *drag_row;
-	gboolean                 interactive;
-};
+typedef struct {
+	EmpathyContactListStore    *store;
+	GtkUIManager               *ui;
+	GtkTreeRowReference        *drag_row;
+	EmpathyContactListFeatures  features;
+} EmpathyContactListViewPriv;
 
 typedef struct {
 	EmpathyContactListView *view;
@@ -136,11 +140,13 @@ static void        contact_list_view_pixbuf_cell_data_func     (GtkTreeViewColum
 								GtkTreeModel               *model,
 								GtkTreeIter                *iter,
 								EmpathyContactListView     *view);
+#ifdef HAVE_VOIP
 static void        contact_list_view_voip_cell_data_func       (GtkTreeViewColumn          *tree_column,
 								GtkCellRenderer            *cell,
 								GtkTreeModel               *model,
 								GtkTreeIter                *iter,
 								EmpathyContactListView     *view);
+#endif
 static void        contact_list_view_avatar_cell_data_func     (GtkTreeViewColumn          *tree_column,
 								GtkCellRenderer            *cell,
 								GtkTreeModel               *model,
@@ -167,23 +173,23 @@ static void        contact_list_view_row_activated_cb          (EmpathyContactLi
 								GtkTreePath                *path,
 								GtkTreeViewColumn          *col,
 								gpointer                    user_data);
+#ifdef HAVE_VOIP
 static void        contact_list_view_voip_activated_cb         (EmpathyCellRendererActivatable *cell,
 								const gchar                *path_string,
 								EmpathyContactListView     *view);
+#endif
 static void        contact_list_view_row_expand_or_collapse_cb (EmpathyContactListView      *view,
 								GtkTreeIter                *iter,
 								GtkTreePath                *path,
 								gpointer                    user_data);
 static void        contact_list_view_action_cb                 (GtkAction                  *action,
 								EmpathyContactListView      *view);
-static void        contact_list_view_action_activated          (EmpathyContactListView      *view,
-								EmpathyContact              *contact);
 static void        contact_list_view_voip_activated            (EmpathyContactListView      *view,
 								EmpathyContact              *contact);
 
 enum {
 	PROP_0,
-	PROP_INTERACTIVE
+	PROP_FEATURES
 };
 
 static const GtkActionEntry entries[] = {
@@ -223,14 +229,16 @@ static const GtkActionEntry entries[] = {
 	  N_("_Send File..."), NULL, N_("Send a file"),
 	  G_CALLBACK (contact_list_view_action_cb)
 	},
-	{ "Log", GTK_STOCK_JUSTIFY_LEFT,
+	{ "Log", EMPATHY_IMAGE_LOG,
 	  N_("_View Previous Conversations"), NULL, N_("View previous conversations with this contact"),
 	  G_CALLBACK (contact_list_view_action_cb)
 	},
+#ifdef HAVE_VOIP
 	{ "Call", EMPATHY_IMAGE_VOIP,
 	  N_("_Call"), NULL, N_("Start a voice or video conversation with this contact"),
 	  G_CALLBACK (contact_list_view_action_cb)
 	},
+#endif
 };
 
 static guint n_entries = G_N_ELEMENTS (entries);
@@ -239,7 +247,9 @@ static const gchar *ui_info =
 	"<ui>"
 	"  <popup name='Contact'>"
 	"    <menuitem action='Chat'/>"
+#ifdef HAVE_VOIP
 	"    <menuitem action='Call'/>"
+#endif
 	"    <menuitem action='Log'/>"
 	"    <menuitem action='SendFile'/>"
 	"    <separator/>"
@@ -252,6 +262,7 @@ static const gchar *ui_info =
 	"  </popup>"
 	"  <popup name='Group'>"
 	"    <menuitem action='Rename'/>"
+	"    <menuitem action='Remove'/>"
 	"  </popup>"
 	"</ui>";
 
@@ -311,17 +322,18 @@ empathy_contact_list_view_class_init (EmpathyContactListViewClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      empathy_marshal_VOID__OBJECT_STRING_STRING,
+			      _empathy_gtk_marshal_VOID__OBJECT_STRING_STRING,
 			      G_TYPE_NONE,
 			      3, EMPATHY_TYPE_CONTACT, G_TYPE_STRING, G_TYPE_STRING);
 
 	g_object_class_install_property (object_class,
-					 PROP_INTERACTIVE,
-					 g_param_spec_boolean ("interactive",
-							       "View is interactive",
-							       "Is the view interactive",
-							       FALSE,
-							       G_PARAM_READWRITE));
+					 PROP_FEATURES,
+					 g_param_spec_flags ("features",
+							     "Features of the view",
+							     "Falgs for all enabled features",
+							      EMPATHY_TYPE_CONTACT_LIST_FEATURES,
+							      0,
+							      G_PARAM_READWRITE));
 
 	g_type_class_add_private (object_class, sizeof (EmpathyContactListViewPriv));
 }
@@ -404,8 +416,8 @@ contact_list_view_get_property (GObject    *object,
 	priv = GET_PRIV (object);
 
 	switch (param_id) {
-	case PROP_INTERACTIVE:
-		g_value_set_boolean (value, priv->interactive);
+	case PROP_FEATURES:
+		g_value_set_flags (value, priv->features);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
@@ -425,8 +437,8 @@ contact_list_view_set_property (GObject      *object,
 	priv = GET_PRIV (object);
 
 	switch (param_id) {
-	case PROP_INTERACTIVE:
-		empathy_contact_list_view_set_interactive (view, g_value_get_boolean (value));
+	case PROP_FEATURES:
+		empathy_contact_list_view_set_features (view, g_value_get_flags (value));
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
@@ -435,40 +447,69 @@ contact_list_view_set_property (GObject      *object,
 }
 
 EmpathyContactListView *
-empathy_contact_list_view_new (EmpathyContactListStore *store)
+empathy_contact_list_view_new (EmpathyContactListStore    *store,
+			       EmpathyContactListFeatures  features)
 {
-	EmpathyContactListViewPriv *priv;
 	EmpathyContactListView     *view;
-	
-	view = g_object_new (EMPATHY_TYPE_CONTACT_LIST_VIEW, NULL);
-	priv = GET_PRIV (view);
+	EmpathyContactListViewPriv *priv;
 
+	g_return_val_if_fail (EMPATHY_IS_CONTACT_LIST_STORE (store), NULL);
+	
+	view = g_object_new (EMPATHY_TYPE_CONTACT_LIST_VIEW,
+			    "features", features,
+			    NULL);
+
+	priv = GET_PRIV (view);
 	priv->store = g_object_ref (store);
-	contact_list_view_setup (view);
+	contact_list_view_setup (EMPATHY_CONTACT_LIST_VIEW (view));
 
 	return view;
 }
 
 void
-empathy_contact_list_view_set_interactive (EmpathyContactListView  *view,
-					   gboolean                 interactive)
+empathy_contact_list_view_set_features (EmpathyContactListView     *view,
+					EmpathyContactListFeatures  features)
 {
 	EmpathyContactListViewPriv *priv = GET_PRIV (view);
 
 	g_return_if_fail (EMPATHY_IS_CONTACT_LIST_VIEW (view));
 
-	priv->interactive = interactive;
-	g_object_notify (G_OBJECT (view), "interactive");
+	priv->features = features;
+
+	/* Update DnD source/dest */
+	if (features & EMPATHY_CONTACT_LIST_FEATURE_CONTACT_DRAG) {
+		gtk_drag_source_set (GTK_WIDGET (view),
+				     GDK_BUTTON1_MASK,
+				     drag_types_source,
+				     G_N_ELEMENTS (drag_types_source),
+				     GDK_ACTION_MOVE | GDK_ACTION_COPY);
+	} else {
+		gtk_drag_source_unset (GTK_WIDGET (view));
+
+	}
+
+	if (features & EMPATHY_CONTACT_LIST_FEATURE_CONTACT_DROP) {
+		gtk_drag_dest_set (GTK_WIDGET (view),
+				   GTK_DEST_DEFAULT_ALL,
+				   drag_types_dest,
+				   G_N_ELEMENTS (drag_types_dest),
+				   GDK_ACTION_MOVE | GDK_ACTION_COPY);
+	} else {
+		/* FIXME: URI could still be  droped depending on FT feature */
+		gtk_drag_dest_unset (GTK_WIDGET (view));
+	}
+
+	g_object_notify (G_OBJECT (view), "features");
 }
 
-gboolean
-empathy_contact_list_view_get_interactive (EmpathyContactListView  *view)
+EmpathyContactListFeatures
+empathy_contact_list_view_get_features (EmpathyContactListView  *view)
 {
 	EmpathyContactListViewPriv *priv = GET_PRIV (view);
 
 	g_return_val_if_fail (EMPATHY_IS_CONTACT_LIST_VIEW (view), FALSE);
 
-	return priv->interactive;
+	return priv->features;
 }
 
 EmpathyContact *
@@ -489,7 +530,9 @@ empathy_contact_list_view_get_selected (EmpathyContactListView *view)
 		return NULL;
 	}
 
-	gtk_tree_model_get (model, &iter, COL_CONTACT, &contact, -1);
+	gtk_tree_model_get (model, &iter,
+			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &contact,
+			    -1);
 
 	return contact;
 }
@@ -514,8 +557,8 @@ empathy_contact_list_view_get_selected_group (EmpathyContactListView *view)
 	}
 
 	gtk_tree_model_get (model, &iter,
-			    COL_IS_GROUP, &is_group,
-			    COL_NAME, &name,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, &is_group,
+			    EMPATHY_CONTACT_LIST_STORE_COL_NAME, &name,
 			    -1);
 
 	if (!is_group) {
@@ -526,48 +569,6 @@ empathy_contact_list_view_get_selected_group (EmpathyContactListView *view)
 	return name;
 }
 
-GtkWidget *
-empathy_contact_list_view_get_group_menu (EmpathyContactListView *view)
-{
-	EmpathyContactListViewPriv *priv;
-	GtkWidget                 *widget;
-
-	g_return_val_if_fail (EMPATHY_IS_CONTACT_LIST_VIEW (view), NULL);
-
-	priv = GET_PRIV (view);
-
-	widget = gtk_ui_manager_get_widget (priv->ui, "/Group");
-
-	return widget;
-}
-
-GtkWidget *
-empathy_contact_list_view_get_contact_menu (EmpathyContactListView *view,
-					    EmpathyContact         *contact)
-{
-	EmpathyLogManager *log_manager;
-	gboolean           can_show_log;
-	gboolean           can_send_file;
-	gboolean           can_voip;
-
-	g_return_val_if_fail (EMPATHY_IS_CONTACT_LIST_VIEW (view), NULL);
-	g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), NULL);
-
-	log_manager = empathy_log_manager_new ();
-	can_show_log = empathy_log_manager_exists (log_manager,
-						   empathy_contact_get_account (contact),
-						   empathy_contact_get_id (contact),
-						   FALSE);
-	g_object_unref (log_manager);
-	can_send_file = FALSE;
-	can_voip = empathy_contact_can_voip (contact);
-
-	return contact_list_view_get_contact_menu (view,
-						   can_send_file,
-						   can_show_log,
-						   can_voip);
-}
-
 static void
 contact_list_view_setup (EmpathyContactListView *view)
 {
@@ -620,12 +621,13 @@ contact_list_view_setup (EmpathyContactListView *view)
 		view, NULL);
 
 	gtk_tree_view_column_add_attribute (col, cell,
-					    "name", COL_NAME);
+					    "name", EMPATHY_CONTACT_LIST_STORE_COL_NAME);
 	gtk_tree_view_column_add_attribute (col, cell,
-					    "status", COL_STATUS);
+					    "status", EMPATHY_CONTACT_LIST_STORE_COL_STATUS);
 	gtk_tree_view_column_add_attribute (col, cell,
-					    "is_group", COL_IS_GROUP);
+					    "is_group", EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP);
 
+#ifdef HAVE_VOIP
 	/* Voip Capability Icon */
 	cell = empathy_cell_renderer_activatable_new ();
 	gtk_tree_view_column_pack_start (col, cell, FALSE);
@@ -641,6 +643,7 @@ contact_list_view_setup (EmpathyContactListView *view)
 	g_signal_connect (cell, "path-activated",
 			  G_CALLBACK (contact_list_view_voip_activated_cb),
 			  view);
+#endif
 
 	/* Avatar */
 	cell = gtk_cell_renderer_pixbuf_new ();
@@ -679,21 +682,6 @@ contact_list_view_setup (EmpathyContactListView *view)
 		drag_atoms_source[i] = gdk_atom_intern (drag_types_source[i].target,
 							FALSE);
 	}
-
-	/* Note: We support the COPY action too, but need to make the
-	 * MOVE action the default.
-	 */
-	gtk_drag_source_set (GTK_WIDGET (view),
-			     GDK_BUTTON1_MASK,
-			     drag_types_source,
-			     G_N_ELEMENTS (drag_types_source),
-			     GDK_ACTION_MOVE | GDK_ACTION_COPY);
-
-	gtk_drag_dest_set (GTK_WIDGET (view),
-			   GTK_DEST_DEFAULT_ALL,
-			   drag_types_dest,
-			   G_N_ELEMENTS (drag_types_dest),
-			   GDK_ACTION_MOVE | GDK_ACTION_COPY);
 }
 
 static void
@@ -702,12 +690,13 @@ contact_list_view_row_has_child_toggled_cb (GtkTreeModel          *model,
 					    GtkTreeIter           *iter,
 					    EmpathyContactListView *view)
 {
+	EmpathyContactListViewPriv *priv = GET_PRIV (view);
 	gboolean  is_group = FALSE;
 	gchar    *name = NULL;
 
 	gtk_tree_model_get (model, iter,
-			    COL_IS_GROUP, &is_group,
-			    COL_NAME, &name,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, &is_group,
+			    EMPATHY_CONTACT_LIST_STORE_COL_NAME, &name,
 			    -1);
 
 	if (!is_group || G_STR_EMPTY (name)) {
@@ -715,7 +704,8 @@ contact_list_view_row_has_child_toggled_cb (GtkTreeModel          *model,
 		return;
 	}
 
-	if (empathy_contact_group_get_expanded (name)) {
+	if (!(priv->features & EMPATHY_CONTACT_LIST_FEATURE_GROUPS_SAVE) ||
+	    empathy_contact_group_get_expanded (name)) {
 		g_signal_handlers_block_by_func (view,
 						 contact_list_view_row_expand_or_collapse_cb,
 						 GINT_TO_POINTER (TRUE));
@@ -878,8 +868,7 @@ contact_list_view_drag_motion (GtkWidget      *widget,
 		dm->view = EMPATHY_CONTACT_LIST_VIEW (widget);
 		dm->path = gtk_tree_path_copy (path);
 
-		dm->timeout_id = g_timeout_add (
-			1500,
+		dm->timeout_id = g_timeout_add_seconds (1,
 			(GSourceFunc) contact_list_view_drag_motion_cb,
 			dm);
 	}
@@ -940,7 +929,6 @@ contact_list_view_drag_data_get (GtkWidget        *widget,
 	const gchar                *contact_id;
 	const gchar                *account_id;
 	gchar                      *str;
-	
 
 	priv = GET_PRIV (widget);
 
@@ -1020,27 +1008,21 @@ contact_list_view_cell_set_background (EmpathyContactListView *view,
 
 	style = gtk_widget_get_style (GTK_WIDGET (view));
 
-	if (!is_group) {
-		if (is_active) {
-			color = style->bg[GTK_STATE_SELECTED];
+	if (!is_group && is_active) {
+		color = style->bg[GTK_STATE_SELECTED];
 
-			/* Here we take the current theme colour and add it to
-			 * the colour for white and average the two. This
-			 * gives a colour which is inline with the theme but
-			 * slightly whiter.
-			 */
-			color.red = (color.red + (style->white).red) / 2;
-			color.green = (color.green + (style->white).green) / 2;
-			color.blue = (color.blue + (style->white).blue) / 2;
+		/* Here we take the current theme colour and add it to
+		 * the colour for white and average the two. This
+		 * gives a colour which is inline with the theme but
+		 * slightly whiter.
+		 */
+		color.red = (color.red + (style->white).red) / 2;
+		color.green = (color.green + (style->white).green) / 2;
+		color.blue = (color.blue + (style->white).blue) / 2;
 
-			g_object_set (cell,
-				      "cell-background-gdk", &color,
-				      NULL);
-		} else {
-			g_object_set (cell,
-				      "cell-background-gdk", NULL,
-				      NULL);
-		}
+		g_object_set (cell,
+			      "cell-background-gdk", &color,
+			      NULL);
 	} else {
 		g_object_set (cell,
 			      "cell-background-gdk", NULL,
@@ -1060,9 +1042,9 @@ contact_list_view_pixbuf_cell_data_func (GtkTreeViewColumn     *tree_column,
 	gboolean  is_active;
 
 	gtk_tree_model_get (model, iter,
-			    COL_IS_GROUP, &is_group,
-			    COL_IS_ACTIVE, &is_active,
-			    COL_ICON_STATUS, &icon_name,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, &is_group,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_ACTIVE, &is_active,
+			    EMPATHY_CONTACT_LIST_STORE_COL_ICON_STATUS, &icon_name,
 			    -1);
 
 	g_object_set (cell,
@@ -1075,6 +1057,7 @@ contact_list_view_pixbuf_cell_data_func (GtkTreeViewColumn     *tree_column,
 	contact_list_view_cell_set_background (view, cell, is_group, is_active);
 }
 
+#ifdef HAVE_VOIP
 static void
 contact_list_view_voip_cell_data_func (GtkTreeViewColumn      *tree_column,
 				       GtkCellRenderer        *cell,
@@ -1087,9 +1070,9 @@ contact_list_view_voip_cell_data_func (GtkTreeViewColumn      *tree_column,
 	gboolean can_voip;
 
 	gtk_tree_model_get (model, iter,
-			    COL_IS_GROUP, &is_group,
-			    COL_IS_ACTIVE, &is_active,
-			    COL_CAN_VOIP, &can_voip,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, &is_group,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_ACTIVE, &is_active,
+			    EMPATHY_CONTACT_LIST_STORE_COL_CAN_VOIP, &can_voip,
 			    -1);
 
 	g_object_set (cell,
@@ -1099,6 +1082,7 @@ contact_list_view_voip_cell_data_func (GtkTreeViewColumn      *tree_column,
 
 	contact_list_view_cell_set_background (view, cell, is_group, is_active);
 }
+#endif
 
 static void
 contact_list_view_avatar_cell_data_func (GtkTreeViewColumn     *tree_column,
@@ -1113,10 +1097,10 @@ contact_list_view_avatar_cell_data_func (GtkTreeViewColumn     *tree_column,
 	gboolean   is_active;
 
 	gtk_tree_model_get (model, iter,
-			    COL_PIXBUF_AVATAR, &pixbuf,
-			    COL_PIXBUF_AVATAR_VISIBLE, &show_avatar,
-			    COL_IS_GROUP, &is_group,
-			    COL_IS_ACTIVE, &is_active,
+			    EMPATHY_CONTACT_LIST_STORE_COL_PIXBUF_AVATAR, &pixbuf,
+			    EMPATHY_CONTACT_LIST_STORE_COL_PIXBUF_AVATAR_VISIBLE, &show_avatar,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, &is_group,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_ACTIVE, &is_active,
 			    -1);
 
 	g_object_set (cell,
@@ -1143,9 +1127,9 @@ contact_list_view_text_cell_data_func (GtkTreeViewColumn     *tree_column,
 	gboolean show_status;
 
 	gtk_tree_model_get (model, iter,
-			    COL_IS_GROUP, &is_group,
-			    COL_IS_ACTIVE, &is_active,
-			    COL_STATUS_VISIBLE, &show_status,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, &is_group,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_ACTIVE, &is_active,
+			    EMPATHY_CONTACT_LIST_STORE_COL_STATUS_VISIBLE, &show_status,
 			    -1);
 
 	g_object_set (cell,
@@ -1166,8 +1150,8 @@ contact_list_view_expander_cell_data_func (GtkTreeViewColumn     *column,
 	gboolean is_active;
 
 	gtk_tree_model_get (model, iter,
-			    COL_IS_GROUP, &is_group,
-			    COL_IS_ACTIVE, &is_active,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, &is_group,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_ACTIVE, &is_active,
 			    -1);
 
 	if (gtk_tree_model_iter_has_child (model, iter)) {
@@ -1195,25 +1179,100 @@ contact_list_view_get_contact_menu (EmpathyContactListView *view,
 				    gboolean               can_show_log,
 				    gboolean               can_voip)
 {
-	EmpathyContactListViewPriv *priv;
-	GtkAction                 *action;
-	GtkWidget                 *widget;
+	EmpathyContactListViewPriv *priv = GET_PRIV (view);
+	GtkAction                  *action;
 
-	priv = GET_PRIV (view);
+	if (!(priv->features & (EMPATHY_CONTACT_LIST_FEATURE_CONTACT_CHAT |
+				EMPATHY_CONTACT_LIST_FEATURE_CONTACT_CALL |
+				EMPATHY_CONTACT_LIST_FEATURE_CONTACT_LOG |
+				EMPATHY_CONTACT_LIST_FEATURE_CONTACT_FT |
+				EMPATHY_CONTACT_LIST_FEATURE_CONTACT_INVITE |
+				EMPATHY_CONTACT_LIST_FEATURE_CONTACT_EDIT |
+				EMPATHY_CONTACT_LIST_FEATURE_CONTACT_INFO |
+				EMPATHY_CONTACT_LIST_FEATURE_CONTACT_REMOVE))) {
+		return NULL;
+	}
 
-	/* Sort out sensitive items */
-	action = gtk_ui_manager_get_action (priv->ui, "/Contact/Log");
-	gtk_action_set_sensitive (action, can_show_log);
+	/* Sort out sensitive/visible items */
+	action = gtk_ui_manager_get_action (priv->ui, "/Contact/Chat");
+	gtk_action_set_visible (action, priv->features & EMPATHY_CONTACT_LIST_FEATURE_CONTACT_CHAT);
 
+#ifdef HAVE_VOIP
 	action = gtk_ui_manager_get_action (priv->ui, "/Contact/Call");
 	gtk_action_set_sensitive (action, can_voip);
+	gtk_action_set_visible (action, priv->features & EMPATHY_CONTACT_LIST_FEATURE_CONTACT_CALL);
+#endif
+
+	action = gtk_ui_manager_get_action (priv->ui, "/Contact/Log");
+	gtk_action_set_sensitive (action, can_show_log);
+	gtk_action_set_visible (action, priv->features & EMPATHY_CONTACT_LIST_FEATURE_CONTACT_LOG);
+
 
 	action = gtk_ui_manager_get_action (priv->ui, "/Contact/SendFile");
-	gtk_action_set_visible (action, can_send_file);
+	gtk_action_set_visible (action, can_send_file && (priv->features & EMPATHY_CONTACT_LIST_FEATURE_CONTACT_FT));
 
-	widget = gtk_ui_manager_get_widget (priv->ui, "/Contact");
+	action = gtk_ui_manager_get_action (priv->ui, "/Contact/Invite");
+	gtk_action_set_visible (action, priv->features & EMPATHY_CONTACT_LIST_FEATURE_CONTACT_INVITE);
 
-	return widget;
+	action = gtk_ui_manager_get_action (priv->ui, "/Contact/Edit");
+	gtk_action_set_visible (action, priv->features & EMPATHY_CONTACT_LIST_FEATURE_CONTACT_EDIT);
+
+	action = gtk_ui_manager_get_action (priv->ui, "/Contact/Information");
+	gtk_action_set_visible (action, priv->features & EMPATHY_CONTACT_LIST_FEATURE_CONTACT_INFO);
+
+	action = gtk_ui_manager_get_action (priv->ui, "/Contact/Remove");
+	gtk_action_set_visible (action, priv->features & EMPATHY_CONTACT_LIST_FEATURE_CONTACT_REMOVE);
+
+	return gtk_ui_manager_get_widget (priv->ui, "/Contact");
+}
+
+GtkWidget *
+empathy_contact_list_view_get_group_menu (EmpathyContactListView *view)
+{
+	EmpathyContactListViewPriv *priv = GET_PRIV (view);
+	GtkAction                  *action;
+
+	g_return_val_if_fail (EMPATHY_IS_CONTACT_LIST_VIEW (view), NULL);
+
+	if (!(priv->features & (EMPATHY_CONTACT_LIST_FEATURE_GROUPS_RENAME |
+				EMPATHY_CONTACT_LIST_FEATURE_GROUPS_REMOVE))) {
+		return NULL;
+	}
+
+	action = gtk_ui_manager_get_action (priv->ui, "/Group/Rename");
+	gtk_action_set_visible (action, priv->features & EMPATHY_CONTACT_LIST_FEATURE_GROUPS_RENAME);
+
+	action = gtk_ui_manager_get_action (priv->ui, "/Group/Remove");
+	gtk_action_set_visible (action, priv->features & EMPATHY_CONTACT_LIST_FEATURE_GROUPS_REMOVE);
+
+	return gtk_ui_manager_get_widget (priv->ui, "/Group");
+}
+
+GtkWidget *
+empathy_contact_list_view_get_contact_menu (EmpathyContactListView *view,
+					    EmpathyContact         *contact)
+{
+	EmpathyLogManager *log_manager;
+	gboolean           can_show_log;
+	gboolean           can_send_file;
+	gboolean           can_voip;
+
+	g_return_val_if_fail (EMPATHY_IS_CONTACT_LIST_VIEW (view), NULL);
+	g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), NULL);
+
+	log_manager = empathy_log_manager_new ();
+	can_show_log = empathy_log_manager_exists (log_manager,
+						   empathy_contact_get_account (contact),
+						   empathy_contact_get_id (contact),
+						   FALSE);
+	g_object_unref (log_manager);
+	can_send_file = FALSE;
+	can_voip = empathy_contact_can_voip (contact);
+
+	return contact_list_view_get_contact_menu (view,
+						   can_send_file,
+						   can_show_log,
+						   can_voip);
 }
 
 static gboolean
@@ -1232,7 +1291,7 @@ contact_list_view_button_press_event_cb (EmpathyContactListView *view,
 
 	priv = GET_PRIV (view);
 
-	if (!priv->interactive || event->button != 3) {
+	if (event->button != 3) {
 		return FALSE;
 	}
 
@@ -1255,7 +1314,9 @@ contact_list_view_button_press_event_cb (EmpathyContactListView *view,
 	gtk_tree_model_get_iter (model, &iter, path);
 	gtk_tree_path_free (path);
 
-	gtk_tree_model_get (model, &iter, COL_CONTACT, &contact, -1);
+	gtk_tree_model_get (model, &iter,
+			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &contact,
+			    -1);
 
 	if (contact) {
 		menu = empathy_contact_list_view_get_contact_menu (view, contact);
@@ -1288,21 +1349,24 @@ contact_list_view_row_activated_cb (EmpathyContactListView *view,
 	GtkTreeModel               *model;
 	GtkTreeIter                 iter;
 
-	if (!priv->interactive) {
+	if (!(priv->features & EMPATHY_CONTACT_LIST_FEATURE_CONTACT_CHAT)) {
 		return;
 	}
 
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (view));
 
 	gtk_tree_model_get_iter (model, &iter, path);
-	gtk_tree_model_get (model, &iter, COL_CONTACT, &contact, -1);
+	gtk_tree_model_get (model, &iter,
+			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &contact,
+			    -1);
 
 	if (contact) {
-		contact_list_view_action_activated (view, contact);
+		empathy_chat_with_contact (contact);
 		g_object_unref (contact);
 	}
 }
 
+#ifdef HAVE_VOIP
 static void
 contact_list_view_voip_activated_cb (EmpathyCellRendererActivatable *cell,
 				     const gchar                    *path_string,
@@ -1313,7 +1377,7 @@ contact_list_view_voip_activated_cb (EmpathyCellRendererActivatable *cell,
 	GtkTreeIter                 iter;
 	EmpathyContact             *contact;
 
-	if (!priv->interactive) {
+	if (!(priv->features & EMPATHY_CONTACT_LIST_FEATURE_CONTACT_CALL)) {
 		return;
 	}
 
@@ -1322,13 +1386,16 @@ contact_list_view_voip_activated_cb (EmpathyCellRendererActivatable *cell,
 		return;
 	}
 
-	gtk_tree_model_get (model, &iter, COL_CONTACT, &contact, -1);
+	gtk_tree_model_get (model, &iter,
+			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &contact,
+			    -1);
 
 	if (contact) {
 		contact_list_view_voip_activated (view, contact);
 		g_object_unref (contact);
 	}
 }
+#endif
 
 
 static void
@@ -1337,14 +1404,19 @@ contact_list_view_row_expand_or_collapse_cb (EmpathyContactListView *view,
 					     GtkTreePath           *path,
 					     gpointer               user_data)
 {
-	GtkTreeModel *model;
-	gchar        *name;
-	gboolean      expanded;
+	EmpathyContactListViewPriv *priv = GET_PRIV (view);
+	GtkTreeModel               *model;
+	gchar                      *name;
+	gboolean                    expanded;
+
+	if (!(priv->features & EMPATHY_CONTACT_LIST_FEATURE_GROUPS_SAVE)) {
+		return;
+	}
 
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (view));
 
 	gtk_tree_model_get (model, iter,
-			    COL_NAME, &name,
+			    EMPATHY_CONTACT_LIST_STORE_COL_NAME, &name,
 			    -1);
 
 	expanded = GPOINTER_TO_INT (user_data);
@@ -1377,16 +1449,16 @@ contact_list_view_action_cb (GtkAction             *action,
 	parent = empathy_get_toplevel_window (GTK_WIDGET (view));
 
 	if (contact && strcmp (name, "Chat") == 0) {
-		contact_list_view_action_activated (view, contact);
+		empathy_chat_with_contact (contact);
 	}
 	else if (contact && strcmp (name, "Call") == 0) {
 		contact_list_view_voip_activated (view, contact);
 	}
 	else if (contact && strcmp (name, "Information") == 0) {
-		empathy_contact_information_dialog_show (contact, parent, FALSE);
+		empathy_contact_information_dialog_show (contact, parent, FALSE, FALSE);
 	}
 	else if (contact && strcmp (name, "Edit") == 0) {
-		empathy_contact_information_dialog_show (contact, parent, TRUE);
+		empathy_contact_information_dialog_show (contact, parent, TRUE, FALSE);
 	}
 	else if (contact && strcmp (name, "Remove") == 0) {
 		/* FIXME: Ask for confirmation */
@@ -1408,6 +1480,12 @@ contact_list_view_action_cb (GtkAction             *action,
 	}
 	else if (group && strcmp (name, "Rename") == 0) {
 	}
+	else if (group && strcmp (name, "Remove") == 0) {
+		EmpathyContactList *list;
+
+		list = empathy_contact_list_store_get_list_iface (priv->store);
+		empathy_contact_list_remove_group (list, group);
+	}
 
 	g_free (group);
 	if (contact) {
@@ -1415,35 +1493,10 @@ contact_list_view_action_cb (GtkAction             *action,
 	}
 }
 
-static void
-contact_list_view_action_activated (EmpathyContactListView *view,
-				    EmpathyContact         *contact)
-{
-	MissionControl *mc;
-
-	mc = empathy_mission_control_new ();
-	mission_control_request_channel (mc,
-					 empathy_contact_get_account (contact),
-					 TP_IFACE_CHANNEL_TYPE_TEXT,
-					 empathy_contact_get_handle (contact),
-					 TP_HANDLE_TYPE_CONTACT,
-					 NULL, NULL);
-	g_object_unref (mc);
-}
-
 static void
 contact_list_view_voip_activated (EmpathyContactListView *view,
 				  EmpathyContact         *contact)
 {
-	MissionControl *mc;
-
-	mc = empathy_mission_control_new ();
-	mission_control_request_channel (mc,
-					 empathy_contact_get_account (contact),
-					 TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA,
-					 empathy_contact_get_handle (contact),
-					 TP_HANDLE_TYPE_CONTACT,
-					 NULL, NULL);
-	g_object_unref (mc);
+	empathy_call_with_contact (contact);
 }
 
