@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2007 Collabora Ltd.
+ * Copyright (C) 2007-2008 Collabora Ltd.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -223,6 +223,23 @@ empathy_contact_widget_get_contact (GtkWidget *widget)
 	return information->contact;
 }
 
+void
+empathy_contact_widget_set_contact (GtkWidget      *widget,
+				    EmpathyContact *contact)
+{
+	EmpathyContactWidget *information;
+
+	g_return_if_fail (GTK_IS_WIDGET (widget));
+	g_return_if_fail (EMPATHY_IS_CONTACT (contact));
+
+	information = g_object_get_data (G_OBJECT (widget), "EmpathyContactWidget");
+	if (!information) {
+		return;
+	}
+
+	contact_widget_set_contact (information, contact);
+}
+
 void
 empathy_contact_widget_set_account_filter (GtkWidget                       *widget,
 					   EmpathyAccountChooserFilterFunc  filter,
@@ -289,6 +306,12 @@ static void
 contact_widget_set_contact (EmpathyContactWidget *information,
 			    EmpathyContact        *contact)
 {
+	if (contact == information->contact ||
+	    (contact && information->contact &&
+	     empathy_contact_equal (contact, information->contact))) {
+		return;
+	}
+
 	contact_widget_remove_contact (information);
 	if (contact) {
 		information->contact = g_object_ref (contact);
@@ -406,6 +429,9 @@ contact_widget_contact_update (EmpathyContactWidget *information)
 		g_signal_connect_swapped (information->contact, "notify::presence",
 					  G_CALLBACK (contact_widget_presence_notify_cb),
 					  information);
+		g_signal_connect_swapped (information->contact, "notify::presence-message",
+					  G_CALLBACK (contact_widget_presence_notify_cb),
+					  information);
 		g_signal_connect_swapped (information->contact, "notify::avatar",
 					  G_CALLBACK (contact_widget_avatar_notify_cb),
 					  information);
