@@ -1,5 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
+ * Copyright (C) 2003-2007 Imendio AB
  * Copyright (C) 2007 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or
@@ -27,7 +28,7 @@
 
 #include <libmissioncontrol/mc-account.h>
 
-#include "gossip-message.h"
+#include "empathy-message.h"
 
 G_BEGIN_DECLS
 
@@ -64,7 +65,7 @@ EmpathyLogManager *empathy_log_manager_new                   (void);
 void               empathy_log_manager_add_message           (EmpathyLogManager *manager,
 							      const gchar       *chat_id,
 							      gboolean           chatroom,
-							      GossipMessage     *message);
+							      EmpathyMessage     *message);
 gboolean           empathy_log_manager_exists                (EmpathyLogManager *manager,
 							      McAccount         *account,
 							      const gchar       *chat_id,
@@ -73,6 +74,8 @@ GList *            empathy_log_manager_get_dates             (EmpathyLogManager
 							      McAccount         *account,
 							      const gchar       *chat_id,
 							      gboolean           chatroom);
+GList *            empathy_log_manager_get_messages_for_file (EmpathyLogManager *manager,
+							      const gchar       *filename);
 GList *            empathy_log_manager_get_messages_for_date (EmpathyLogManager *manager,
 							      McAccount         *account,
 							      const gchar       *chat_id,
@@ -82,6 +85,8 @@ GList *            empathy_log_manager_get_last_messages     (EmpathyLogManager
 							      McAccount         *account,
 							      const gchar       *chat_id,
 							      gboolean           chatroom);
+GList *            empathy_log_manager_get_messages_for_file (EmpathyLogManager *manager,
+							      const gchar       *filename);
 GList *            empathy_log_manager_get_chats             (EmpathyLogManager *manager,
 							      McAccount         *account);
 GList *            empathy_log_manager_search_new            (EmpathyLogManager *manager,
