@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2006 Xavier Claessens <xclaesse@gmail.com>
- * Copyright (C) 2007 Collabora Ltd.
+ * Copyright (C) 2007-2008 Collabora Ltd.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -501,7 +501,7 @@ empathy_tp_group_class_init (EmpathyTpGroupClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      empathy_marshal_VOID__OBJECT_OBJECT_UINT_STRING,
+			      _empathy_marshal_VOID__OBJECT_OBJECT_UINT_STRING,
 			      G_TYPE_NONE,
 			      4, EMPATHY_TYPE_CONTACT, EMPATHY_TYPE_CONTACT, G_TYPE_UINT, G_TYPE_STRING);
 
@@ -511,7 +511,7 @@ empathy_tp_group_class_init (EmpathyTpGroupClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      empathy_marshal_VOID__OBJECT_OBJECT_UINT_STRING,
+			      _empathy_marshal_VOID__OBJECT_OBJECT_UINT_STRING,
 			      G_TYPE_NONE,
 			      4, EMPATHY_TYPE_CONTACT, EMPATHY_TYPE_CONTACT, G_TYPE_UINT, G_TYPE_STRING);
 
@@ -521,7 +521,7 @@ empathy_tp_group_class_init (EmpathyTpGroupClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      empathy_marshal_VOID__OBJECT_OBJECT_UINT_STRING,
+			      _empathy_marshal_VOID__OBJECT_OBJECT_UINT_STRING,
 			      G_TYPE_NONE,
 			      4, EMPATHY_TYPE_CONTACT, EMPATHY_TYPE_CONTACT, G_TYPE_UINT, G_TYPE_STRING);
 
@@ -531,7 +531,7 @@ empathy_tp_group_class_init (EmpathyTpGroupClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      empathy_marshal_VOID__OBJECT_OBJECT_UINT_STRING,
+			      _empathy_marshal_VOID__OBJECT_OBJECT_UINT_STRING,
 			      G_TYPE_NONE,
 			      4, EMPATHY_TYPE_CONTACT, EMPATHY_TYPE_CONTACT, G_TYPE_UINT, G_TYPE_STRING);
 
@@ -562,8 +562,8 @@ empathy_tp_group_new (McAccount *account,
 	DBusGProxy         *group_iface;
 	GError             *error = NULL;
 
-	g_return_val_if_fail (TELEPATHY_IS_CHAN (tp_chan), NULL);
 	g_return_val_if_fail (MC_IS_ACCOUNT (account), NULL);
+	g_return_val_if_fail (TELEPATHY_IS_CHAN (tp_chan), NULL);
 
 	group_iface = tp_chan_get_interface (tp_chan,
 					     TP_IFACE_QUARK_CHANNEL_INTERFACE_GROUP);
@@ -620,10 +620,12 @@ tp_group_async_cb (DBusGProxy *proxy, GError *error, gpointer user_data)
 }
 
 void
-empathy_tp_group_close  (EmpathyTpGroup *group)
+empathy_tp_group_close (EmpathyTpGroup *group)
 {
 	EmpathyTpGroupPriv *priv = GET_PRIV (group);
 
+	g_return_if_fail (EMPATHY_IS_TP_GROUP (group));
+
 	tp_chan_close_async (DBUS_G_PROXY (priv->tp_chan),
 			     tp_group_async_cb,
 			     "Failed to close");
@@ -679,6 +681,9 @@ empathy_tp_group_add_member (EmpathyTpGroup *group,
 	GArray             *handles;
 	guint               handle;
 
+	g_return_if_fail (EMPATHY_IS_TP_GROUP (group));
+	g_return_if_fail (EMPATHY_IS_CONTACT (contact));
+
 	handle = empathy_contact_get_handle (contact);
 	handles = g_array_new (FALSE, FALSE, sizeof (guint));
 	g_array_append_val (handles, handle);
@@ -722,6 +727,9 @@ empathy_tp_group_remove_member (EmpathyTpGroup *group,
 	GArray             *handles;
 	guint               handle;
 
+	g_return_if_fail (EMPATHY_IS_TP_GROUP (group));
+	g_return_if_fail (EMPATHY_IS_CONTACT (contact));
+
 	handle = empathy_contact_get_handle (contact);
 	handles = g_array_new (FALSE, FALSE, sizeof (guint));
 	g_array_append_val (handles, handle);
@@ -740,6 +748,8 @@ empathy_tp_group_get_members (EmpathyTpGroup *group)
 {
 	EmpathyTpGroupPriv *priv = GET_PRIV (group);
 
+	g_return_val_if_fail (EMPATHY_IS_TP_GROUP (group), NULL);
+
 	g_list_foreach (priv->members, (GFunc) g_object_ref, NULL);
 
 	return g_list_copy (priv->members);
@@ -751,6 +761,8 @@ empathy_tp_group_get_local_pendings (EmpathyTpGroup *group)
 	EmpathyTpGroupPriv *priv = GET_PRIV (group);
 	GList              *pendings = NULL, *l;
 
+	g_return_val_if_fail (EMPATHY_IS_TP_GROUP (group), NULL);
+
 	for (l = priv->local_pendings; l; l = l->next) {
 		EmpathyPendingInfo *info;
 		EmpathyPendingInfo *new_info;
@@ -770,6 +782,8 @@ empathy_tp_group_get_remote_pendings (EmpathyTpGroup *group)
 {
 	EmpathyTpGroupPriv *priv = GET_PRIV (group);
 
+	g_return_val_if_fail (EMPATHY_IS_TP_GROUP (group), NULL);
+
 	g_list_foreach (priv->remote_pendings, (GFunc) g_object_ref, NULL);
 
 	return g_list_copy (priv->remote_pendings);
@@ -814,12 +828,27 @@ empathy_tp_group_get_object_path (EmpathyTpGroup *group)
 	return dbus_g_proxy_get_path (DBUS_G_PROXY (priv->tp_chan));
 }
 
+TpChan *
+empathy_tp_group_get_channel (EmpathyTpGroup *group)
+{
+	EmpathyTpGroupPriv *priv;
+
+	g_return_val_if_fail (EMPATHY_IS_TP_GROUP (group), NULL);
+
+	priv = GET_PRIV (group);
+
+	return priv->tp_chan;
+}
+
 gboolean
 empathy_tp_group_is_member (EmpathyTpGroup *group,
 			    EmpathyContact *contact)
 {
 	EmpathyTpGroupPriv *priv = GET_PRIV (group);
 
+	g_return_val_if_fail (EMPATHY_IS_TP_GROUP (group), FALSE);
+	g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), FALSE);
+
 	return g_list_find (priv->members, contact) != NULL;
 }
 
